require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/basic_group/basic_group_attributes'
require 'uu/os/basic_group/basic_group_create'
require 'uu/os/basic_group/basic_group_get_artifact_interface_list_item'
require 'uu/os/basic_group/basic_group_get_artifact_interface_list'
require 'uu/os/basic_group/basic_group_get_basic_group_list_item'
require 'uu/os/basic_group/basic_group_get_basic_group_list'
require 'uu/os/basic_group/basic_group_set_attributes'
require 'uu/os/basic_group/basic_group_add_artifact_interface'
require 'uu/os/basic_group/basic_group_remove_artifact_interface'

# Module Unicorn Universe.
module UU
  # Module Operating System.
  module OS
    # Basic group service.
    module BasicGroup
      extend self

      # Service path
      PATH = 'ues/core/role/UESBasicGroup'
      
      # Creates a new basic group in a specified location. If the meta artifact is not selected in BasicGroupCreate object,
      # the new basic group is created by Basic Group meta artifact from uuOS application. Competent role for the new basic group
      # is always the same role as the competent role for the organizational unit of newly created basic group.
      #
      # @param [String, UU::OS::UESURI] location_uri UESURI of the folder / organizational unit / uuApp meta model where the basic group will be created.
      # @param [BasicGroupCreate] basic_group Object containing the attributes of the basic group.
      # @option basic_group [String, UU::OS::BasicGroup::BasicGroupType] :basic_group_type Specifies the type of basic group. There can be only one basic group for each of the AUTHORITIES, AUDITORS, EXECUTIVES and READERS types in the organizational unit of newly created basic group. If nil or empty the create command fails. Selected basic group type cannot be changed after the basic group creation.
      # @option basic_group [String] :name Name of the new basic group. If nil or empty, name is generated according to the selected basic group type.
      # @option basic_group [String] :code Code of the new basic group. If nil or empty, code is generated according to the selected basic group type.
      # @option basic_group [String] :description Description of the new basic group. If nil is set, tDescription from meta artifact is used if set, otherwise no description is set on the new basic group.
      # @option basic_group [String, UU::OS::Artifact::SecurityLevel] :security_level Security level. If nil is set, security level is taken from the meta artifact tSecurityLevel. If the meta artifact tSecurityLevel is higher than the user’s security clearance, the create command fails.
      # @option basic_group [Boolean] :searchable_in_territory Specifies if it is possible to search the new basic group in its territory. If nil is set, tSearchableInTerritory from meta artifact is used if set, otherwise the create command fails.
      # @option basic_group [String, UU::OS::UESURI] :icon_uri URI of the basic group’s icon. If nil is set, the group uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option basic_group [Boolean] :dry_run Specifies whether this is only a test run to verify that the object can be created. The default value is false.
      # @option basic_group [Boolean] :component_content Defines whether the primarily displayed content of the created BasicGroup is component content. Default value is false.
      #
      # @return [UU::OS::UESURI] UESURI of the newly created basic group.
      # @example
      #   # Create a basic group
      #   UU::OS::BasicGroup.create('ues:TERRITORY:LOCATION',
      #      :basic_group_type => 'EXECUTIVES'
      #   )
      def create(location_uri, basic_group = nil)
        svc = UU::OS::REST::RemoteClient.new(BasicGroup)
        payload = UU::OS::BasicGroup::BasicGroupCreate.new(basic_group).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Deletes the specified basic group role from the system. The basic group is deleted even if it contains a locked sheet or attachment.
      # The basic group cannot be deleted if: the basic group is in an active cast, the basic group's workflow is not created, a property
      # contains a reference to this basic group, a time sheet exists with a reference to this basic group, the basic_group_type attribute's
      # value is "AUTHORITIES" or "AUDITORS".
      #
      # @param [String, UU::OS::UESURI] basic_group_uri UESURI of the basic group which is about to be deleted.
      #
      # @example
      #   # Command that deletes basic group
      #   UU::OS::BasicGroup.delete(basic_group_uri)
      #
      def delete(basic_group_uri)
        svc = UU::OS::REST::RemoteClient.new(BasicGroup)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', basic_group_uri)
        end
      end

      # Command for setting attributes of a basic group. The command cannot modify references to objects like location, competent role and so on.
      #
      # @param [String, UU::OS::UESURI] basic_group_uri UESURI of the basic group whose attributes will be modified.
      # @param [BasicGroupSetAttributes] basic_group Object containing the changed attributes of the basic group.
      # @option basic_group [String] :name The new name of the basic group. If nil is set, the name is not modified.
      # @option basic_group [String] :code The new code the basic group. If nil is set, the code is not modified. The code must be unique in a given territory, otherwise the set_attributes command fails.
      # @option basic_group [String] :description The new description of the basic group. If nil is set, the description is not modified.
      # @option basic_group [String, UU::OS::Artifact::SecurityLevel] :security_level The new security level of the basic group. If nil is set, the security level is not modified. If the given security level is higher than the user’s security clearance, the create command fails.
      # @option basic_group [Boolean] :searchable_in_territory Specifies if it is possible to search the basic group in its territory. If nil is set, this attribute is not modified.
      # @option basic_group [String, UU::OS::UESURI] :icon_uri URI of the basic group’s icon. If nil is set, the icon is not modified. If nil URI ('ues:{[-1]}:{[-1]}:') is set, the group icon is removed and the group uses icon from meta artifact. Allowed format of icon URI: ‘ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{CODE_OF_ICON}’. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option basic_group [Boolean] :component_content Defines whether the primarily displayed content of the created BasicGroup is component content. Default value is false.
      #
      # @return [UU::OS::UESURI] Returns the UESURI of the basic group.
      #
      # @example
      #   # Set basic group attributes
      #   UU::OS::BasicGroup.set_attributes('ues:TERRITORY:BASIC_GROUP',
      #      :name => 'New Name',
      #      :code => 'NEW_CODE',
      #      :description => 'Some sample description',
      #      :searchable_in_territory => false
      #   )
      #
      def set_attributes(basic_group_uri, basic_group = nil)
        svc = UU::OS::REST::RemoteClient.new(BasicGroup)
        payload = UU::OS::BasicGroup::BasicGroupSetAttributes.new(basic_group).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', basic_group_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns attributes representing the basic group specified by the basic_group_uri parameter.
      # The command does not change the state of the basic group in the system.
      #
      # @param [String, UU::OS::UESURI] basic_group_uri UESURI of the basic group whose attributes will be returned.
      #
      # @return [BasicGroupAttributes] # Returns Object containing the attributes of specified basic group.
      #
      def get_attributes(basic_group_uri)
        svc = UU::OS::REST::RemoteClient.new(BasicGroup)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', basic_group_uri)
          return BasicGroup::BasicGroupAttributes.new(res)
        end
      end

      # Creates new artifact interface connection to the specified role.
      #
      # @param [String, UU::OS::UESURI] basic_group_uri UESURI of the basic group where the artifact interface will be connected.
      # @param [BasicGroupAddArtifactInterface] artifact_interface Object containing the changed attributes of the basic group.
      #
      # @option artifact_interface [String] :artifact_interface_code Code of the artifact interface that is to be connected to the specified basic group. If nil is set, the command fails.
      #
      # @return [UU::OS::UESURI] Returns the UESURI of the basic group.
      #
      # @example
      #   # Add an artifact interface
      #   UU::OS::BasicGroup.add_artifact_interface('ues:TERRITORY:BASIC_GROUP',
      #      :artifact_interface_code => 'UU.OS/READ_WRITE'
      #   )
      #
      def add_artifact_interface(basic_group_uri, artifact_interface = nil)
        svc = UU::OS::REST::RemoteClient.new(BasicGroup)
        payload = UU::OS::BasicGroup::BasicGroupAddArtifactInterface.new(artifact_interface).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('addArtifactInterface', basic_group_uri, payload)
        end
      end

      # Removes role interface connection from the specified group role. Only connected interfaces can be disconnected from the group role.
      #
      # @param [String, UU::OS::UESURI] basic_group_uri UESURI of the basic group where the artifact interface will be disconnected.
      # @param [BasicGroupRemoveArtifactInterface] artifact_interface Object containing the artifact interface code that is to be disconnected.
      #
      # @option artifact_interface [String] :artifact_interface_code Code of the artifact interface that is to be disconnected from the specified basic group. If nil is set, the command fails.
      #
      # @example
      #   # Remove an artifact interface
      #   UU::OS::BasicGroup.remove_artifact_interface('ues:TERRITORY:BASIC_GROUP',
      #      :artifact_interface_code => 'UU.OS/READ_WRITE'
      #   )
      #
      def remove_artifact_interface(basic_group_uri, artifact_interface = nil)
        svc = UU::OS::REST::RemoteClient.new(BasicGroup)
        payload = UU::OS::BasicGroup::BasicGroupRemoveArtifactInterface.new(artifact_interface).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('removeArtifactInterface', basic_group_uri, payload)
        end
      end

      # Returns sorted list of the codes of artifact interfaces that are connected to the specified basic group.
      #
      # @param [String, UU::OS::UESURI] basic_group_uri UESURI of the basic group for which the codes of connected artifact interfaces are to be listed.
      #
      # @return [UU::OS::REST::ResultList<BasicGroup::BasicGroupGetArtifactInterfaceList, BasicGroup::BasicGroupGetArtifactInterfaceListItem>] Returns the list of objects containing the codes of artifact interfaces.
      #
      def get_artifact_interface_list(basic_group_uri)
        svc = UU::OS::REST::RemoteClient.new(BasicGroup)

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getArtifactInterfaceList', basic_group_uri)
          return UU::OS::REST::ResultList.new(BasicGroup::BasicGroupGetArtifactInterfaceList, BasicGroup::BasicGroupGetArtifactInterfaceListItem, res)
        end
      end

      # Returns list of the basic groups which are located in the organizational unit of specified artifact. The list of the returned
      # basic groups can be filtered and ordered arbitrarily. The parameters available for customized filtering and ordering are the following:
      # name, code, basic_group_type. The list is sorted by name of the basic group by default.
      #
      # @param [String, UU::OS::UESURI] artifact_uri UESURI of an existing artifact. Basic groups from the organizational unit where the artifact is located will be returned.
      # @param [BasicGroupGetBasicGroupList] criteria DTO containing criteria for listing access role interfaces.
      # @option criteria [String] :query The <b>UES query</b> string. It has to conform the UESQuery syntax (see {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation} for more info). Filtering and order of the result list can be specified there. If nil is set, no filtering is applied and entries are ordered by name of casted subjects (and code as the second criteria).
      #
      # @return [UU::OS::REST::ResultList<BasicGroup::GetBasicGroupList, BasicGroup::GetBasicGroupListItem>] Returns the list of basic groups which are located in the same organizational unit as specified artifact.
      #
      def get_basic_group_list(artifact_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(BasicGroup)
        dto = BasicGroup::BasicGroupGetBasicGroupList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getBasicGroupList', artifact_uri)
          return UU::OS::REST::ResultList.new(BasicGroup::BasicGroupGetBasicGroupList, BasicGroup::BasicGroupGetBasicGroupListItem, res)
        end
      end
    end
  end
end