require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/permission/permission_get_authority_data'
require 'uu/os/permission/permission_get_authority_list_item'
require 'uu/os/permission/permission_get_authority_list'
require 'uu/os/permission/permission_get_mask_attributes'
require 'uu/os/permission/permission_get_mask_list'
require 'uu/os/permission/permission_get_system_group_attributes'
require 'uu/os/permission/permission_get_system_group_list'
require 'uu/os/permission/permission_grant'
require 'uu/os/permission/permission_revoke'
require 'uu/os/permission/permission_authorize'
require 'uu/os/permission/permission_authorize_result'
require 'base64'

# Module Unicorn Universe.
module UU
  # Module Operating System.
  module OS
    # This service allows managing permissions for objects (only Artifacts in current version).
    # Permissions provide access to Use Cases (Set State, Create Property, ...) of objects (Artifact, Property, ...).
    # It is possible to manage permissions according to specified mask or according to detailed permission specification.
    # For more information see {uuOS Access Rights - Guideline}[https://plus4u.net/ues/sesm?SessFree=ues%253A%255B210795%255D%253A%255B44191580132632900%255D%253A].
    module Permission
      extend self

      # REST service url path.
      PATH = 'ues/core/security/UESPermission'
      # Grants permissions on objects (only Artifacts in current version) according to given Permission Mask
      # or according to detailed permission specification (defined access level for chosen use cases).
      # It is possible to grant permission on objects for any allowed authority (Role, Group or System Group). <br>
      # To list all allowed system groups use uuCommand {#get_system_group_list}. <br>
      # All possible levels may be found as constants in {UU::OS::Permission::PermissionLevel} class.
      #
      # @param [String, UU::OS::UESURI] entity_uri URI of the object for which permissions will be granted. Must not be nil.
      # @param [UU::OS::Permission::PermissionGrant] permission Specific options.
      # @option permission [String, UU::OS::UESURI] :authority_uri URI of the authority for which permissions will be granted. Must not be nil.
      # @option permission [String, File, UU::OS::REST::BinaryValue] :data <b>Deprecated.</b> Information about permission settings in JSON format. The permission is specified by the use case and the level.
      # @option permission [String, UU::OS::UESURI] :mask_uri URI of the Permission mask according that the permission will be set. Option :data has higher priority than this one.
      #
      # @example
      #   # Prepare data for the permission in JSON format.
      #   grant_data = '[{"useCaseCode":"AU_103129","castLevel":"$GUEST"}]'
      #
      #   # Grant permissions for Group or Role according to detailed permission specification.
      #   UU::OS::Permission.grant('ues:TERRITORY:ARTIFACT', :authority_uri => 'ues:TERRITORY:ROLE', :data => grant_data)
      #
      #   # Grant permissions for System Group according to Permission mask.
      #   UU::OS::Permission.grant('ues:TERRITORY:ARTIFACT', :authority_uri => 'ues:${system}:COMPETENT_FOR_ART_IFC', :mask_uri => "ues:TERRITORY:ARTIFACT:MASK")
      #
      #   => nil
      def grant(entity_uri, permission)
        svc = UU::OS::REST::RemoteClient.new(Permission)
        payload = UU::OS::Permission::PermissionGrant.new(permission).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('grant', entity_uri, payload)
        end
      end

      # Revokes permissions on an object (only Artifact in current version) for given Use Cases.
      # It is possible to revoke permissions on objects for any allowed authority (Role, Group or System Group). <br>
      # To list all allowed system groups use uuCommand {#get_system_group_list}.
      # @param [String, UU::OS::UESURI] entity_uri URI of the object for which the permissions will be revoked. Must not be nil.
      # @param [UU::OS::Permission::PermissionRevoke] permission Specific options.
      # @option permission [String, UU::OS::UESURI] :authority_uri URI of the authority for which the permissions will be revoked. Must not be nil.
      # @option permission [String, File, UU::OS::REST::BinaryValue] :data <b>Deprecated.</b> List of Use Cases (in JSON format) for which the permission will be revoked.
      # @option permission [Boolean] :all When true is set, permissions will be revoked for all Use Cases (e.g. whole Permission Mask). Default value is false. Option :data has higher priority than this one.
      #
      # @example
      #   # Revoke permissions on all Use Cases (e.g. Permission mask) for System Group.
      #   UU::OS::Permission.revoke('ues:TERRITORY:ARTIFACT', :authority_uri => 'ues:${system}:COMPETENT_FOR_ART_IFC', :all => true)
      #
      #   # Revoke permissions on all Use Cases (e.g. Permission mask) for Group or Role.
      #   UU::OS::Permission.revoke('ues:TERRITORY:ARTIFACT', :authority_uri => 'ues:TERRITORY:ROLE', :all => true)
      #
      #   # Revoke permissions on specified Use Cases for Group or Role.
      #   revoke_data = '[{"useCaseCode":"AU_103129"},{"useCaseCode":"AU_103300"}]'
      #   UU::OS::Permission.revoke('ues:TERRITORY:ARTIFACT', :authority_uri => 'ues:TERRITORY:ROLE', :data => revoke_data)
      #
      #   => nil
      def revoke(entity_uri, permission)
        svc = UU::OS::REST::RemoteClient.new(Permission)
        payload = UU::OS::Permission::PermissionRevoke.new(permission).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('revoke', entity_uri, payload)
        end
      end

      # Returns a list of authorities (Roles, Groups or System Groups) who have been granted permissions on a object (only Artifacts in current version).
      #
      # @param [String, UU::OS::UESURI] entity_uri URI of the object for which to list all authorities with granted permissions.
      # @param [UU::OS::Permission::PermissionGetAuthorityList] criteria Specific options.
      # @option criteria [String] :query Query for filtering and ordering of the result list (more information in {uuQuery - Guideline}[https://plus4u.net/ues/sesm?SessFree=ues%253AUNI-BT%255B210795%255D%253AUES.DOC%252FCMD_V1%252FUESQUERY%255B44191574519623267%255D%253A]). It is possible to filter the result list by authorityUri and authorityName.
      #
      # @return [Array] An Array with the command result. Contains instances of {UU::OS::Permission::PermissionGetAuthorityListItem}.
      #
      # @example
      #   # Get a list of all authorities which have been granted permissions for a Artifact.
      #   UU::OS::Permission.get_authority_list('ues:TERRITORY:ARTIFACT')
      #
      #   # Get a list of all authorities with authorityName that starts with string John.
      #   UU::OS::Permission.get_authority_list('ues:TERRITORY:ARTIFACT', :query => "authorityName='John%'")
      #
      #   => { "UU::OS::REST::ResultList" : {
      #      "total_size" : "1" ,
      #      ...
      #      "page_entries" : [ {
      #         "UU::OS::Permission::PermissionGetAuthorityListItem" : {
      #            "authorityName" : "John Role" ,
      #            "authorityUri" : "ues:TERRITORY:JOHN_ROLE",
      #            ...
      #            "maskName" : "Read",
      #            "maskUri" : "ues:TERRITORY:META_ARTIFACT:MASK"
      #         }
      #      } ]
      #   } }
      def get_authority_list(entity_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Permission)
        dto = Permission::PermissionGetAuthorityList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAuthorityList', entity_uri)
          return UU::OS::REST::ResultList.new(Permission::PermissionGetAuthorityList, Permission::PermissionGetAuthorityListItem, res)
        end
      end

      # Returns data about permissions granted to the given authority (Role, Group or System Group) on object (only Artifacts in current version). <br>
      # To list all authorities with granted permissions use uuCommand {#get_authority_list}.
      #
      # @param [String, UU::OS::UESURI] entity_uri URI of the object for which the data about permission settings will be returned.
      # @param [UU::OS::Permission::PermissionGetAuthorityData] options Specific options.
      # @option options [String, UU::OS::UESURI] :authority_uri URI of the authority whose granted permission settings will be returned. Must not be nil.
      # @option options [String] :requested_mime_type MIME type of the returned value. Possible values are 'application/json' for JSON and 'text/csv' for CSV. JSON is returned by default.
      #
      # @return [UU::OS::REST::BinaryValue] Data about permission settings for given object and authority.
      #
      # @example
      #   # Get permissions granted to Group or Role in JSON format.
      #   UU::OS::Permission.get_authority_data('ues:TERRITORY:ARTIFACT', :authority_uri => 'ues:TERRITORY:ROLE')
      #
      #   # Get permissions granted to System Group in JSON format.
      #   UU::OS::Permission.get_authority_data('ues:TERRITORY:ARTIFACT', :authority_uri => 'ues:${system}:COMPETENT_FOR_ART_IFC')
      #
      #   # Get permissions granted to Group or Role in CSV format.
      #   UU::OS::Permission.get_authority_data('ues:TERRITORY:ARTIFACT', :authority_uri => 'ues:TERRITORY:ROLE', :requested_mime_type => 'text/csv')
      def get_authority_data(entity_uri, options)
        svc = UU::OS::REST::RemoteClient.new(Permission)
        dto = UU::OS::Permission::PermissionGetAuthorityData.new(options)
        svc.add_parameter('authorityUri', dto.authority_uri)
        if dto.requested_mime_type
          svc.add_parameter('requestedMimeType', dto.requested_mime_type)
        end

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAuthorityData', entity_uri)
          return UU::OS::REST::BinaryValue.new(res, true)
        end
      end

      # Returns a list of permission masks for an Artifact.
      #
      # @param [String, UU::OS::UESURI] artifact_uri URI of the artifact whose permission masks will be listed.
      # @param [UU::OS::Permission::PermissionGetMaskList] criteria Specific options.
      # @option criteria [String] :query Query for filtering and ordering of the result list (more information in {uuQuery - Guideline}[https://plus4u.net/ues/sesm?SessFree=ues%253AUNI-BT%255B210795%255D%253AUES.DOC%252FCMD_V1%252FUESQUERY%255B44191574519623267%255D%253A]). It is possible to filter the result list by name and code.
      #
      # @return [Array] An Array with the command result. Contains instances of {UU::OS::Permission::PermissionGetMaskAttributes}.
      #
      # @example
      #   # Get a list of all permission masks whose name starts with string "Create" and code starts with string "CREATE".
      #   UU::OS::Permission.get_mask_list('ues:TERRITORY:ARTIFACT', :query => "name='Create%' AND code='CREATE%'")
      def get_mask_list(artifact_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Permission)
        dto = Permission::PermissionGetMaskList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getMaskList', artifact_uri)
          return UU::OS::REST::ResultList.new(Permission::PermissionGetMaskList, Permission::PermissionGetMaskAttributes, res)
        end
      end

      # Returns a list of System Groups of an entity (object or pack). It may be used to set permissions for these groups.
      #
      # @param [String, UU::OS::UESURI] entity_uri URI of the object. Must not be nil.
      #
      # @return [Array] An Array with the command result. Contains instances of {UU::OS::Permission::PermissionGetSystemGroupAttributes}.
      #
      # @example
      #   # Get a list of all System Groups for given entity.
      #   UU::OS::Permission.get_system_group_list('ues:TERRITORY:ARTIFACT')
      def get_system_group_list(entity_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Permission)
        dto = Permission::PermissionGetSystemGroupList.new(criteria)

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getSystemGroupList', entity_uri)
          return UU::OS::REST::ResultList.new(Permission::PermissionGetSystemGroupList, Permission::PermissionGetSystemGroupAttributes, res)
        end
      end

      # Checks whether the logged user is authorized to the specific use case over the specific entity. If the user is not authorized to run the specified use case, the authorization exception (UU::OS::Security::AuthorizationException: E05401.M08) is thrown.
      # 
      # Note that authorization entities may differ from main entities of specific uuCommands. 
      # For example the uuCommand UU::OS::Property.create uses an artifact as the main entity, 
      # but it is authorized over the property group where the property is to be created.
      # 
      # @param [String, UU::OS::UESURI] entity_uri <b>Not nil.</b> URI of the entity (artifact, object) over which the logged user will be authorized.
      # @param [UU::OS::Permission::PermissionAuthorize] options Specific options.
      # @option options [String] :use_case_code <b>Not nil.</b> Code of the specific use case for which it will be checked whether the logged user is authorized. Codes of system use cases can be obtained by exporting the access right settings template of the given meta artifact. Note that codes for specific use cases may differ in some uuCommon Apps.
      #
      # @return [UU::OS::Permission::PermissionAuthorizeResult] Object containing the permission authorization attributes.
      #
      # @example
      #   # Checks whether the logged user is authorized to the use case with code AU_101010 over the given entity.
      #   UU::OS::Permission.authorize('ues:TERRITORY:ENTITY', :use_case_code => 'AU_101010')
      def authorize(entity_uri, options)
        svc = UU::OS::REST::RemoteClient.new(Permission)
        dto = Permission::PermissionAuthorize.new(options)

        svc.add_parameter('useCaseCode', dto.use_case_code)

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('authorize', entity_uri)
          return Permission::PermissionAuthorizeResult.new(res)
        end
      end

    end
  end
end
